
/* Chrysalide - Outil d'analyse de fichiers binaires
 * core.c - enregistrement des fonctions principales
 *
 * Copyright (C) 2022 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "core.h"


#include "items/datasize.h"
#include "items/uint.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : space = espace de noms à composer.                           *
*                                                                             *
*  Description : Inscrit les principales fonctions dans l'espace racine.      *
*                                                                             *
*  Retour      : Bilan des enregistrements effectués.                         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool populate_main_scan_namespace(GScanNamespace *space)
{
    bool result;

    result = true;

#define REGISTER_FUNC(s, f)                                                 \
    ({                                                                      \
        bool __result;                                                      \
        __result = g_scan_namespace_register_item(s, G_REGISTERED_ITEM(f)); \
        g_object_unref(G_OBJECT(f));                                        \
        __result;                                                           \
    })

    if (result) result = REGISTER_FUNC(space, g_datasize_function_new());
    //if (result) result = REGISTER_FUNC(space, g_datasize_function_new(), "filesize"); /* Alias */

    if (result) result = REGISTER_FUNC(space, g_uint_function_new(MDS_8_BITS_UNSIGNED));
    if (result) result = REGISTER_FUNC(space, g_uint_function_new(MDS_16_BITS_UNSIGNED));
    if (result) result = REGISTER_FUNC(space, g_uint_function_new(MDS_32_BITS_UNSIGNED));
    if (result) result = REGISTER_FUNC(space, g_uint_function_new(MDS_64_BITS_UNSIGNED));

    return result;

}
