
/* Chrysalide - Outil d'analyse de fichiers binaires
 * literal.h - prototypes pour la représentation d'une valeur concrète
 *
 * Copyright (C) 2022 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_EXPRS_LITERAL_H
#define _ANALYSIS_SCAN_EXPRS_LITERAL_H


#include <regex.h>


#include "../expr.h"
#include "../../../common/szstr.h"



#define G_TYPE_LITERAL_EXPRESSION            g_literal_expression_get_type()
#define G_LITERAL_EXPRESSION(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_LITERAL_EXPRESSION, GLiteralExpression))
#define G_IS_LITERAL_EXPRESSION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_LITERAL_EXPRESSION))
#define G_LITERAL_EXPRESSION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_LITERAL_EXPRESSION, GLiteralExpressionClass))
#define G_IS_LITERAL_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_LITERAL_EXPRESSION))
#define G_LITERAL_EXPRESSION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_LITERAL_EXPRESSION, GLiteralExpressionClass))


/* Expression portant une valeur concrète (instance) */
typedef struct _GLiteralExpression GLiteralExpression;

/* Expression portant une valeur concrète (classe) */
typedef struct _GLiteralExpressionClass GLiteralExpressionClass;


/* Indique le type défini pour un appel de fonction enregistrée. */
GType g_literal_expression_get_type(void);

/* Organise un appel de fonction avec ses arguments. */
GScanExpression *g_literal_expression_new(ExprValueType, ...);

/* Indique la valeur portée par une expression booléenne. */
bool g_literal_expression_get_boolean_value(const GLiteralExpression *, bool *);

/* Indique la valeur portée par une expression d'entier. */
bool g_literal_expression_get_integer_value(const GLiteralExpression *, unsigned long long *);

/* Indique la valeur portée par une expression de chaîne. */
bool g_literal_expression_get_string_value(const GLiteralExpression *, const sized_string_t **);

/* Indique la valeur portée par une expression rationnelle. */
bool g_literal_expression_get_regex_value(const GLiteralExpression *, const regex_t **);



#endif  /* _ANALYSIS_SCAN_EXPRS_LITERAL_H */
